/*! For license information please see taipy-gui-dom.js.LICENSE.txt */
!function(){var e={745:function(e,r,t){"use strict";var o=t(822);r.s=o.createRoot,o.hydrateRoot},251:function(e,r,t){"use strict";var o=t(255),n=Symbol.for("react.element"),s=(Symbol.for("react.fragment"),Object.prototype.hasOwnProperty),i=o.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentOwner,u={key:!0,ref:!0,__self:!0,__source:!0};r.jsx=function(e,r,t){var o,c={},f=null,p=null;for(o in void 0!==t&&(f=""+t),void 0!==r.key&&(f=""+r.key),void 0!==r.ref&&(p=r.ref),r)s.call(r,o)&&!u.hasOwnProperty(o)&&(c[o]=r[o]);if(e&&e.defaultProps)for(o in r=e.defaultProps)void 0===c[o]&&(c[o]=r[o]);return{$$typeof:n,type:e,key:f,ref:p,props:c,_owner:i.current}}},893:function(e,r,t){"use strict";e.exports=t(251)},822:function(e,r,t){e.exports=t(912)(3935)},255:function(e,r,t){e.exports=t(912)(7294)},912:function(e){"use strict";e.exports=TaipyGuiDependencies}},r={};function t(o){var n=r[o];if(void 0!==n)return n.exports;var s=r[o]={exports:{}};return e[o](s,s.exports,t),s.exports}!function(){"use strict";var e=t(893),r=t(745),o=TaipyGui,n=document.getElementById("root");n&&(0,r.s)(n).render((0,e.jsx)(o.Router,{}))}()}();